;;; RACE/A Example: 2-AFC ;;;
;;;
;;; The model describes a simple 2-alternative forced choice experiment
;;; that demonstrates how RACE/A works within then cognitive architecture ACT-R.
;;; The model presses a button with its left index finger if an "A" appears on
;;; the screen, and with its right index finger if a B appears. 
;;; You can set :race-enabled to nil to see how default ACT-R would behave.
;;;
;;;
;;; More information on RACE/A can be found onhttp:// www.ai.rug.nl/~leendert/race
;;; 
;;; Leendert van Maanen, July 26 2009 ;;;

(defvar *response* nil)
(defvar *done*  nil)
(defvar *response-time* nil)
(defvar *allresp* nil)
(defvar *allrt* nil)

(defun do-exp (n str)
;; runs a sequence of trials. n can be a positive number, str can be either "A" or "B"
  (setf *allresp* nil)
  (setf *allrt* nil)
  (let ((x 0))
    (loop until (>= x n) do
          (incf x 1)
          (do-trial str)
          ))
  (format t "mean ACC: ~a~%" (cond 
                              ((equalp str "A") (perc "f" *allresp*))
                              ((equalp str "B") (perc "j" *allresp*))))
  (format t "mean RT: ~a" (avg *allrt*)))

(defun do-trial (str)
;; runs one trial. str can be either "A" or "B"
  (reset)
  (let ((window (open-exp-window "2-AFC Experiment"                                      
                                 :visible nil
                                 :width 300
                                 :height 300
                                 :x 300
                                 :y 300)))
    
    (add-text-to-exp-window :text str
                            :width 30
                            :height 30
                            :x 145
                            :y 150)
    (install-device window)
    (setf *response* nil)
    (setf *response-time* nil)
    
    (when *actr-enabled-p*
      (proc-display)
      (run 5))
    (while (null *done*) 
      (allow-event-manager window))
    (clear-exp-window)
    (proc-display))
  (push *response* *allresp*)
  (push *response-time* *allrt*))


(defmethod rpm-window-key-event-handler ((win rpm-window) key)
;; to catch the response
  (setf *response-time* (get-time))
  (setf *response* (string key))
  (setf *done* t)
)

;;;; analysis tools ;;;;
(defun perc (resp all)
  (let ((s 0.0))
    (dolist (i all)
      (when (equalp i resp) (incf s 1)))
    (/ s (length all))))

(defun avg (numbers)
  (if (null numbers)
      (error "Average of the empty list is undefined.")
      (float (/ (reduce #'+ numbers)
         (length numbers)))))

(clear-all)

(define-model 2afc

(sgp :v nil
     :trace-detail low     
     :esc t
     :rt -2 ; because RACE/A uses the threshold chunk to compute the decision ratio (Van Maanen, 2009) this parameter influences the RACE/A process as well
     :lf .1
     :ans 0.3
     :visual-attention-latency 0

     :race-enabled nil     ; RACE/A is enabled or not
     :race-ratio .968     ; the Luce ratio value above which a chunk is retrieved
     :race-freq 200       ; update frequency (Hz)
     :race-trace nil      ; extra information on the activation in the trace. Enabling this option creates a very long trace
     :race-a .8           ; decay scaling parameter 
     :race-b 17.0         ; spreading activation scaling parameter
     :race-saliency nil   ; influence of perceptual information
     :race-default-acc 0  ; if a chunk does not have a baselevel activation, it will be set to this value
)

(chunk-type goal state)
(chunk-type lemma value)
(chunk-type concept value)
(chunk-type srmapping value hand)

(add-dm (lemma isa chunk)
        (left isa chunk)
        (right isa chunk)
        (Alemma isa lemma value "A")
        (Blemma isa lemma value "B")
        (Aleft isa srmapping value "A" hand left)
        (Bright isa srmapping value "B" hand right)

        (goal isa goal state "start")) 

(set-base-levels
 (Alemma .5)
 (Blemma .5)
 (Aleft 1)
 (Bright 1)
)

(add-sji
 (Aleft Alemma 1)
 (Bright Blemma 1)
)

(P attend-stim
   =goal>
      isa         goal
      state       "start"
   ?visual>
      state       free
==>
   +goal>
      state       "identify"
   +visual-location>
      isa         visual-location
      :attended   nil
      screen-x    lowest
      screen-y    lowest
)

(P identify-stim
   =goal>
      isa         goal
      state       "identify"
   =visual-location>
      isa         visual-location
   ?visual>
      state       free
==>
   +visual>
      isa         move-attention
      screen-pos  =visual-location
)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(P retrieve-lemma
   =visual>
      isa         text
      value       =val
==>
   +retrieval>
      ISA         lemma
      value       =val
)
  
(P retrieve-response-race ; this production only fires if RACE/A is enabled
   =retrieval>
      isa         lemma
      value       =val
   !eval!         (race-enabled (get-module race))
==>
   +retrieval>
      ISA         srmapping
      ;value       =val
)

(P retrieve-response-norace ; this production only fires if RACE/A is disabled
  =retrieval>
      isa         lemma
      value       =val
   !eval!         (not (race-enabled (get-module race)))
==>
   +retrieval>
      ISA         srmapping
      value       =val
)

(P respond
   ?manual>
      state       free
   =retrieval>
      isa srmapping
      value       =val
      hand        =hand
==>
   +manual>
      isa         punch
      hand        =hand
      finger      index
      !output!    =val
   -goal>
)

(goal-focus goal)

(setf *actr-enabled-p* t)
)

